if(!dojo._hasResource["com.ibm.mm.builder.utils.themeUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.utils.themeUtil"] = true;
/**
 * @author Jarod (zhangpy@cn.ibm.com)
 */
dojo.provide("com.ibm.mm.builder.utils.themeUtil");

com.ibm.mm.builder.utils.themeUtil.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.builder.utils.themeUtil");
/**
 * return theme model
 * @param {Object} theme
 */
com.ibm.mm.builder.utils.themeUtil.getThemeModel = function(refresh) {
    if (refresh || !this.themeModel) {
        this.themeModel = com.ibm.mashups.enabler.model.Factory.getThemeModel();
        this.themeModel.setStrategy(new com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy(25));
    }
    return this.themeModel;
};

com.ibm.mm.builder.utils.themeUtil.getThemeNode = function(themeId) {
    var themeNode = this.getThemeModel().find(themeId).start();
    if (themeNode) { return themeNode; }
    
    try {
        // when the theme is invalid, set the page theme as default theme and commit.
        var defaultThemeId = this.getDefaultThemeId();
        var navigationModel = com.ibm.mm.builder.utils.navigationUtil.getNavigationModel();
        var pageId = com.ibm.mm.builder.utils.pageUtil.getCurrentPageId();
        var page = navigationModel.find(pageId).start();
        page.setTheme(defaultThemeId);
        navigationModel.commit().start();
    } catch (ex) {
        com.ibm.mm.builder.utils.themeUtil.LOGGER.severe("getThemeNode", "Fail to set default theme to current page!");
    }
    return this.getDefaultThemeNode();
};

com.ibm.mm.builder.utils.themeUtil.getDefaultThemeNode = function() {
    var themeModel = this.getThemeModel();
    return themeModel.find(this.getDefaultThemeId()).start();
};

com.ibm.mm.builder.utils.themeUtil.getDefaultThemeId = function(spaceId) {
    // if space root node has set theme, return space' theme
    // add the spaceId check to fix bug 9121
    if (spaceId) {
        var themeId = com.ibm.mm.builder.utils.navigationUtil.getNavigationRootNode(spaceId).getTheme();
        if (themeId && themeId != "null") { return themeId; }
    }
    var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
    return cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.CLIENT_DEFAULT_THEME_ID);
};

com.ibm.mm.builder.utils.themeUtil.getCurrentSpaceThemeId = function() {
    var spaceId = com.ibm.mm.builder.utils.spaceUtil.getCurrentSpaceId();
    var themeId = com.ibm.mm.builder.utils.navigationUtil.getNavigationRootNode(spaceId).getTheme();
    if (themeId && themeId != "null") { return themeId; }
};


/**
 * return theme title
 * @param {Object} theme
 */
com.ibm.mm.builder.utils.themeUtil.getThemeTitle = function(theme, locale) {
    if (!theme) { return null; }
    return com.ibm.mm.builder.utils.enablerUtil.getNodeTitle(theme, locale);
};

/**
 * Load the given page's theme
 * @param {String} pageId
 */
com.ibm.mm.builder.utils.themeUtil.loadTheme = function(pageId) {

    var page = com.ibm.mm.builder.utils.pageUtil.getDefaultPage(pageId);
    if (!page) { //cannot find the any possible page
        this.loadDefaultTheme();
        return;
    }
    var currentThemeId = this.getCurrentThemeId();
    if (!currentThemeId) { // first time to load theme
        this._load(this.getThemeId(pageId));
    }
};

com.ibm.mm.builder.utils.themeUtil.loadDefaultTheme = function() {
    var dt = this.getCurrentSpaceThemeId();
    var ct = this.getCurrentThemeId();
    if (!ct || ct != dt) {
        this._load(dt);
    }
};

com.ibm.mm.builder.utils.themeUtil.getCurrentThemeId = function() {
    var themeId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentTheme().getID();
    return themeId === "null" ? null : themeId;
};

com.ibm.mm.builder.utils.themeUtil.getThemeId = function(page) {
    var pu = com.ibm.mm.builder.utils.pageUtil;
    var pageNode = null;
    if (page) {
		if (dojo.isString(page)) {
			pageNode = pu.getPageNode(page);
		} else {
			pageNode = page;
		}
	} else {
		pageNode = pu.getCurrentPageNode();
	}
    if (!pageNode) { return null; }
    var themeId = pageNode.getTheme();
    if (!themeId || themeId === "null") { return this.getDefaultThemeId(); }
    return themeId;
};

com.ibm.mm.builder.utils.themeUtil.getCurrentThemeNode = function() {
    return this.getThemeNode(this.getCurrentThemeId());
};

com.ibm.mm.builder.utils.themeUtil._load = function(themeId) {
    if (!themeId || themeId === "null") {
        var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
        themeId = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.CLIENT_DEFAULT_THEME_ID);
    }
    var tm = this.getThemeNode(themeId);
    if (!tm) {
        alert(dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_MAIN_LOAD_THEME_MODEL_FAIL_0, [themeId || "null"]));
    }
    com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentTheme().setID(themeId);
    
    //set body id to enable theme developer to customize the widget style
    document.body.id = themeId;
    
    var htmlUrl = this.getThemeModel().findResourceUrl(tm, "theme.html");
    var cssUrl = this.getThemeModel().findResourceUrl(tm, "theme.css");
    com.ibm.mm.builder.utils.htmlUtil.loadCss(cssUrl);
    if (builderConfig.isBidi) {
        var cssRtlUrl = this.getThemeModel().findResourceUrl(tm, "theme_rtl.css");
        com.ibm.mm.builder.utils.htmlUtil.loadCss(cssRtlUrl);
    }
    var bindArgs = {
        url: htmlUrl,
        handle: function(data, ioArgs) {
            ////keep previous children in body
            //var previousChildren = [];
            //dojo.forEach(dojo.body().childNodes, function(item){previousChildren.push(item)});
            //document.body.innerHTML = data;
            //dojo.forEach(previousChildren, "dojo.body().appendChild(item)");
            if (data instanceof Error) { //dojo.body().innerHTML = ioArgs.xhr.responseText;
            return; }
            document.body.innerHTML = data;
            
            var i = 1;
            //handle js if there is
            dojo.forEach(document.body.getElementsByTagName("script"), function(script) {
                //TODO replace enabler internal api with public api.
                try {
                    var id = script.getAttribute("id");
                    if (id === null || id === "") {
                        script.setAttribute("id", "_scr#" + i);
                        i++;
                    }
                    com.ibm.mm.enabler.aggregation.javascript.JAVASCRIPT_HANDLER.handle(script);
                } catch (e) {
                    console.log("Exception evaluate js:" + e.message); //todo, since we're already frozen on tvt.
                }
                
            });
            
            //parse iwidgets in the theme
            var mpFactory = com.ibm.mashups.enabler.io.XHRMultipartFactory;
            var mpHandler = mpFactory.create();
            mpHandler.startTransaction(mpFactory.CONTEXT_ENABLER);
            com.ibm.mm.builder.utils.iwidgetUtil.loadWidgets(null, function() {
                com.ibm.mashups.services.ServiceManager.getService("eventService").broadcastEvent("com.ibm.mashups.builder.switchPage");
            });
            mpHandler.endTransactionDeferred(true).start();
            
            //add theme particular style to body
            var bodyClasses = com.ibm.mm.builder.utils.themeUtil.getCurrentThemeNode().getMetadata("com.ibm.mashups.theme.body.class");
            dojo.forEach(bodyClasses.split(" "), function(className) {
                dojo.addClass(dojo.body(), className);
            });
            
            //add the dbcs classes
            if (ibmConfig.fontSizeEnlarged) {
                dojo.addClass(dojo.body(), com.ibm.mm.builder.constant.dbcsEnlarged);
            }
        },
        error: function(err, ioArgs) {
            //if the theme is wrong, load default theme instead.
            //var u = com.ibm.mm.builder.utils;
            //if (u.pageUtil.getCurrentPageId()) {
            //	u.pageUtil.updatePage(u.pageUtil.getCurrentPageId(), {
            //		theme: u.themeUtil.getDefaultThemeId()
            //	});
            //}
            setTimeout(function() {
                com.ibm.mm.builder.utils.themeUtil.loadDefaultTheme();
            }, 0);
        }
    };
    dojo.xhrGet(bindArgs);
};

com.ibm.mm.builder.utils.themeUtil.unloadAllWidgets = function(needReload) {
    var themeModel = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentTheme();
    var layouts = themeModel.getLayouts();
    var widgetIds = com.ibm.mm.builder.utils.wireUtil.getAllWidgetsInLayouts(layouts);
    
    dojo.forEach(widgetIds, function(widgetId) {
        com.ibm.mm.builder.commonActions.removeWidget(widgetId);
    });
    // needReload indicate whether the unload widgets action require a new load widgets action followed.
    // if we publish this event to enabler, pass in the widget id array, enabler will notify builder to do load widgets action.
    if (needReload) {
        com.ibm.mashups.services.ServiceManager.getService("eventService").publishEvent(com.ibm.mashups.iwidget.Constants.UNLOAD_WIDGETS, widgetIds);
    }
};

mmBuilder_layoutWidgetsPool = [];
com.ibm.mm.builder.utils.themeUtil.unloadWidgetsInLayout = function(layoutId) {
    mmBuilder_layoutWidgetsPool.push(layoutId);
    var themeModel = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentTheme();
    var layouts = themeModel.getLayouts();
    if (layouts) {
		if (mmBuilder_layoutWidgetsPool.length == layouts.length) {
			for (var i = 0; i < layouts.length; i++) {
				mmBuilder_layoutWidgetsPool.pop(layouts[i]);
			}
		}
		if (mmBuilder_layoutWidgetsPool.length === 0) {
			mmBuilder_layoutWidgetsPool = [];
			com.ibm.mm.builder.utils.themeUtil.unloadAllWidgets(true);
		}
	}
};


}
