dojo.provide("com.ibm.pb.dnd.layout.LayoutSource");

dojo.require("com.ibm.dnd.TunedLayoutSource");
dojo.require("com.ibm.pb.dnd.layout.LayoutControlAvatar");
dojo.require("com.ibm.pb.logging.Loggable");
dojo.require("com.ibm.pb.layout.LayoutParser");

dojo.declare("com.ibm.pb.dnd.layout.LayoutSource", [com.ibm.dnd.TunedLayoutSource, com.ibm.pb.logging.Loggable], {
	isDragging: false,
	currentNode: null,
	currentMargin: 0,
	currentTarget: null,
	emptyContainer: null,
	delaySave: true,
	movedItems: [],
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	constructor: function() {
		this.evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.moveWidget = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetMoved", this, "_widgetMoved");
		this.applyDefaultSizing = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.addNewWidget", this, "_widgetDefaultSizingApplied");
	},
	
	managerOverrides: {
		OFFSET_X: 1,
		OFFSET_Y: 1,
		makeAvatar: function(){
			return new com.ibm.pb.dnd.layout.LayoutControlAvatar(this);
		},
		startDrag: function(source, nodes, copy){
			this.source = source;
			this.nodes  = nodes;
			this.copy   = Boolean(copy); // normalizing to true boolean
			this.avatar = this.makeAvatar();
			dojo.body().appendChild(this.avatar.node); //cost 651ms TODO
			dojo.publish("/dnd/start", [source, nodes, this.copy]); 
			this.events = [
				dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"),
				dojo.connect(dojo.doc, "onmouseup",   this, "onMouseUp"),
				dojo.connect(dojo.doc, "onkeydown",   this, "onKeyDown"),
				dojo.connect(dojo.doc, "onkeyup",     this, "onKeyUp"),
				dojo.connect(dojo.doc, "ondragstart",   dojo.stopEvent),
				dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent)
			];
		}
	},
	
	copyState: function(){
		return false;
	},
	onDndStart: function(source, nodes, copy){
		this.inherited(arguments);
		
		if(source !== this) {
			// we need this once only
			return;
		}
		
		this.isDragging = true;
		
		this.currentNode = dojo.byId(nodes[0].id);
		
		// current node stored in controller for use in creator
		ibmCfg.controllers.dnd.currentNode = this.currentNode;
		ibmCfg.controllers.dnd.currentNode._isCanceled = false;
		
		// ghost height stored in controller to use cross-source
		if (ibmCfg.controllers.dnd.ghostHeight == 0) {
			if (dojo.isIE) {
				ibmCfg.controllers.dnd.ghostHeight = 200;
			}
			else {
				ibmCfg.controllers.dnd.ghostHeight = this.currentNode.offsetHeight;
			}
		}
		//if(!dojo.isIE)dojo.style(this.currentNode , {cursor:"move"});	
		
//	this.currentMargin = dojo.marginBox(this.currentNode).w - this.currentNode.offsetWidth;	
		
		var sourceItem = source.getItem(this.currentNode.id);
		if(sourceItem && (sourceItem.type != ibmCfg.controllers.dnd.SHELF_TYPE)){
			if(!dojo.isIE){
				dojo.style(this.currentNode,{display:"none"});
			}
		}
	},
	
	onDrop: function(source, nodes, copy){
		this.LOGGER.entering("onDrop",[source, nodes, copy]);

		if(!this.current && !this.emptyContainer){
			this.onDndCancel();
			return;
		}
		
		var currentNode = dojo.byId(nodes[0].id);
				
		var sourceItem = source.getItem(currentNode.id);
		var type = sourceItem ? sourceItem.type : null;
		
		// is the source from the shelf?
		if(type && (type == ibmCfg.controllers.dnd.SHELF_TYPE)){
			//handle the widget creation from the shelf
			this.onDropFromShelf(source, nodes, copy);
		}else{
			//handle the widget move operation
			if(this != source){
				this.onDropExternal(source, nodes, copy);
			}else{
				this.onDropInternal(nodes, copy);
			}
		}
		
		var currentContainer = null;
		if(this.current){
			currentContainer = this.current.parentNode;
		}else{
			currentContainer = this.emptyContainer;
		}
		
		this._calculateDropTarget(currentNode, currentContainer, type);
		
		this._processDrop(currentNode, currentContainer, source, nodes, copy);

		this.isDragging = false;
		
		this.LOGGER.exiting("onDrop",[source, nodes, copy]);
	},
	
	onDndCancel: function(){
		this.inherited(arguments);
		if (ibmCfg.controllers.dnd.currentNode && !ibmCfg.controllers.dnd.currentNode._isCanceled) {
			if(!dojo.isIE){
				dojo.style(ibmCfg.controllers.dnd.currentNode, {display: ""});
			}
			this.isDragging = false;
			ibmCfg.controllers.dnd.currentNode._isCanceled = true;
		}
	},
	onDropFromShelf: function(source, nodes, copy){
		// Override onDropExternal for the shelf items 
		// so the shelf grid item is not inserted into 
		// the container markup before displaying the widget
		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	_calculateDropTarget: function(currentNode, currentContainer, type){
		if(this.current && (this.current != currentNode) && !this.emptyContainer){
			if(this.before != true){
				// the target is after the current node			
				var controls = dojo.query("."+ibmCfg.controllers.dnd.CONTROL_CLASS, currentContainer);
				for(var i=0;i<controls.length;i++){
					if(controls[i] == this.current){
						//if drop is from shelf, only add 1 to get next control, 
						//otherwise...
						//the portlet node is inserted into the markup and
						//the index needs to add 1 for the node dropped,
						//then add another 1 for the correct node after the target
						var realIndex = (type == "sm") ? i+1 : i+2; 
						if(realIndex < controls.length){
							this.current = controls[realIndex];
							break;
						}else{
							// the target is below the last node set to null to append at end of container
							this.current = null;	
							break;
						}
					}
				}
			}
		}else{
			// the target is an empty container
			if(this.emptyContainer){
				currentContainer = this.emptyContainer;
				this.current = null;
			}else{
				this.onDndCancel();
				return;
			}
		}
	},
	_processDrop: function(currentNode, currentContainer, source, nodes, copy){
		var sourceItem = source.getItem(currentNode.id);		
		if (sourceItem && (sourceItem.type == ibmCfg.controllers.dnd.SHELF_TYPE)) {
			var widget = dijit.byId(nodes[0].id);			
			var nextSiblingId = this.current ? this.layoutParser.extractID(this.current) : null;
			widget.apply(this.layoutParser.extractID(currentContainer), nextSiblingId);
		} else {
			var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
			var lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
			var containerDef = lm.find(this.layoutParser.extractID(currentContainer));
			
			if (containerDef) {
				containerDef.setFinishedCallback(dojo.hitch(this, "_insert"), {
					layoutModel: lm,
					controls: {
						target: this.current,
						current: currentNode
					}
				});
				containerDef.start(false);
			}
			else {
				this.LOGGER.severe("onDrop", "Cannot find container in the layout model");
			}
			
		}
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.widgetMoved", this.layoutParser.extractID(currentNode));
	},
	_insert: function(container, status, x){
		// summary: inserts a control to the new location
		// container: layout container into which to put the new control
		// status: HTTP status returned from request to get container
		// x: {layoutModel, controls {target, current dragging node}}
		this.LOGGER.entering("_insert",[container, status, x]);
		if(status != 200) {
			this.LOGGER.severe("_insert","Could not find container in the layout model.  Cannot move control.",[container, status, x]);
			return;
		}
		if(container instanceof com.ibm.mashups.enabler.layout.LayoutContainer){
			var current = x.layoutModel.find(this.layoutParser.extractID(x.controls.current)).start();
			
			if(!current){
				this.LOGGER.severe("_insert","Cannot find current dragging node in the layout model", [x.controls.current]);
			}
				
			var target = null;
			if (x.controls.target) { 
				target = x.layoutModel.find(this.layoutParser.extractID(x.controls.target)).start();
				if(!target){
					this.LOGGER.severe("_insert","Cannot find target node in the layout model", [x.controls.target]);
				}
			}
	
			if( x.layoutModel.confirmInsert(current,container,target) ){
				if (target && current.getID() == target.getID()) {
					target = null;
				}
				x.layoutModel.insert(current,container,target);
			}
			
			if(typeof ibmc !== "undefined") ibmc.publish("my.setDirtyPage", [ true ]);
			com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
			if(typeof i$ != "undefined") i$.fireEvent("wptheme/contextMenu/invalidate/all");
			
			if(!this.delaySave){
				ibmCfg.controllers.dnd.nm.commit().start();				
			}
			
		} else { 
			this.LOGGER.severe("_insert","Container is not an instance of com.ibm.mashups.enabler.layout.LayoutContainer",[container, status, x]); 
		}
		
		this.LOGGER.exiting("_insert",[container, status, x]);
	},
	
	_widgetMoved: function(iEvent){
		this.syncSourcesMap(iEvent);
	},

	_widgetDefaultSizingApplied: function(iEvent){
		com.ibm.mm.builder.utils.skinUtil.updateDefaultSize(iEvent);
	},

	//This method is used to sync all dnd sources on the page with their map attributes
	syncSourcesMap: function(id){
		
		var nodes = dojo.query(".id-"+id ,this.node);
		if (nodes[0] && nodes[0].id) {
			var nid = nodes[0].id;
			var sources = ibmCfg.controllers.dnd.dndSources;
			for (i in sources) {
				if (i) {
					if (sources[i].getItem(nid)) {
						if (sources[i] != this) {
							this.setItem(nid, sources[i].getItem(nid));
							sources[i].setItem(nid, null);
							sources[i].delItem(nid);
						}
					}
				}
			}
		}
	},
	
	destroy: function(){
		this.evtSvc.unsubscribeEvent(this.moveWidget);
		this.evtSvc.unsubscribeEvent(this.applyDefaultSizing); 
		this.moveWidget = null;
		this.applyDefaultSizing = null;
	}
});
