dojo.provide("com.ibm.portal.fx.Transitions");

dojo.require("dojo.fx");
dojo.require("dojo.NodeList-fx");

com.ibm.portal.fx.Transitions = {
	collapsePortlets: function(/*String*/ portletQuery, /*Object*/args){
		dojo.query(portletQuery || ".wpsPortletBody").wipeOut(dojo.mixin({duration: 1000}, args)).play();
	},
	uncollapsePortlets: function(/*String*/ portletQuery, /*Object*/args){
		dojo.query(portletQuery || ".wpsPortletBody").wipeIn(dojo.mixin({duration: 1000}, args)).play();
	},
	combineMapAnimations: function(/*Object*/ map, /*String?*/ prop) {
		prop = prop || "animation";
		return dojo.fx.combine(dojo.map(map, function(obj){
			return obj.animation;
		}));
	},
	chainMapAnimations: function(/*Object*/ map, /*String?*/ prop) {
		prop = prop || "animation";
		return dojo.fx.chain(dojo.map(map, function(obj){
			return obj.animation;
		}));
	},
	fadeToOverlay: function(/*String*/ query, /*Object*/ args, /*Node?*/ context) {
		// summary: Find all nodes matched by the query and cross-fade them out with an overlay
		//		over top of them which has a class called fxOverlay applied to it to visualize
		//		the old position of the node that was just faded.
		// query: String query used in conjunction with dojo.query to retrieve nodes to animate
		// args: Object animation arguments
		// context: Node optional to use as context from which to execute the dojo.query
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded in by animation
		//			node: Node matched by query that is faded out by animation
		//			animation: dojo._Animation cross-fading animation used to animate the transition
		// 				of the overlay and node.
		// 		} 
		//			
		return dojo.query(query, context).map(function(node) {
			var overlay = dojo.doc.createElement("div");
			var cds = dojo.coords(node);
			dojo.style(overlay, "position", "absolute");
			dojo.style(overlay, "top", (cds.y - dojo.style(node, "marginTop")) + "px");
			dojo.style(overlay, "left", (cds.x - dojo.style(node, "marginLeft")) + "px");
			dojo.style(overlay, "height", cds.h + "px");
			dojo.style(overlay, "width", cds.w + "px");
			dojo.addClass(overlay, "fxOverlay");
			dojo.style(overlay, "border", "solid 2px orange");
			dojo.style(overlay, "opacity", 0);
			dojo.body().appendChild(overlay);
			var anim = dojo.fx.combine([
				dojo.fadeIn(dojo.mixin({node: overlay}, args)),
				dojo.fadeOut(dojo.mixin({node: node}, args))
			]);
			
			return {overlay: overlay, node: node, animation: anim};
		});
	},
	fadeOutOverlay: function(/*Object*/ map, /*Object*/ args) {
		// summary: Cross-fades overlays with the nodes they cover by fading out the overlay
		// 		then fading in the covered node.  Can be used in conjunction with fadeToOverlay.
		//		Note that at the end of each animation, the overlay node will be destroyed.
		// map: Map of overlay nodes and their corresponding nodes that they cover in this format -> 
		// 		{
		//			overlay: Node overlay node
		//			node: Node being covered by overlay
		//		}
		// args: Object animation arguments
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded out by animation
		//			node: Node matched by query that is faded in by animation
		//			animation: dojo._Animation used to cross-fade the overlays with the
		//				nodes they cover.
		// 		}
		return dojo.map(map, function(obj){
			var anim = dojo.fx.combine([
				dojo.fadeOut(dojo.mixin({node: obj.overlay}, args)),
				dojo.fadeIn(dojo.mixin({node: obj.node}, args))
			]);
			return {overlay: obj.overlay, node: obj.node, animation: anim};
		});
	},
	positionOverlay: function(/*Object*/ map, /*Object*/ args) {
		// summary: Slides overlays to the position and size of the nodes they cover.  Can be
		//		used in conjunction with fadeToOverlay.
		// map: Map of overlay nodes and their corresponding nodes that they cover in this format -> 
		// 		{
		//			overlay: Node overlay node to reposition/resize
		//			node: Node being covered by overlay
		//		}
		// args: Object animation arguments
		// returns: Object map of matching nodes in this format ->
		//		{
		//			overlay: Node overlay node which is faded in by animation
		//			node: Node matched by query that is faded out by animation
		//			animation: dojo._Animation used to slide/resize the overlays to the
		//				nodes they cover.
		// 		}
		return dojo.map(map, function(obj){
			var newCds = dojo.coords(obj.node);
			var oldCds = dojo.coords(obj.overlay);
			
			var sizingAnim = dojo.animateProperty(dojo.mixin({ 
				node: obj.overlay,
				properties: {
					width: { start: oldCds.w, end: newCds.w, unit:"px" },
					height: { start: oldCds.h, end: newCds.h, unit:"px" }
				}
			}, args));
			
			var anim = dojo.fx.combine([
				dojo.fx.slideTo(dojo.mixin({node: obj.overlay, unit:"px", left: newCds.l, top: newCds.t}, args)), 
				sizingAnim
			]);
			return {overlay: obj.overlay, node: obj.node, animation: anim};
		});
	},
	showOverlayDisable: function(node,loading){
		// summary: displays an overlay on top of the given node, disabling its contents
		// node: a string for dojo.query that produces nodes to overlay
		// loading: if true, a loading icon will display on the overlay
		var nodes = dojo.query(node); // TODO: a foreach for this array
		var overlays = [];
		for(var i = 0; i < nodes.length; i++){
			nodes[i].style.position = "relative";  // the position of the div must be relative or absolute so the overlay will stay within it
			// create the overlay div
			var overlay = document.createElement("div");
			dojo.addClass(overlay,"ibmPortalOverlayShow");
			if(loading) {
				dojo.addClass(overlay,"ibmPortalLoading");
			}
			// make the overlay the same size as the main content
			dojo.style(overlay,"height",nodes[i].offsetHeight+"px");
			dojo.style(overlay,"width",nodes[i].offsetWidth+"px");
			// save the overlay
			overlays.push(overlay);
			// show the overlay
			nodes[i].appendChild(overlay);
		} 
		return overlays;
	},
	hideOverlayDisable: function(/*DOMNode[]*/overlays){
		// summary: removes an array of overlays by deleting the nodes
		for(var i = 0; i < overlays.length; i++) {
			overlays[i].parentNode.removeChild(overlays[i]);
		}
	}
};
