/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.model.LayoutModel");

dojo.require("com.ibm.mashups.enabler.Identifiable");
dojo.require("com.ibm.mashups.enabler.TreeModelController");
dojo.require("com.ibm.mashups.enabler.Deferred");
dojo.require("com.ibm.mashups.enabler.DeferredIterator");
dojo.require("com.ibm.mashups.enabler.layout.Constants");
dojo.require("com.ibm.mashups.enabler.layout.LayoutNode");
dojo.require("com.ibm.mashups.enabler.layout.LayoutContainer");
dojo.require("com.ibm.mashups.enabler.layout.LayoutControl");
dojo.require("com.ibm.mashups.enabler.layout.LayoutFragment");
dojo.require("com.ibm.mashups.enabler.wire.WireModelProvider");
//dojo..require("com.ibm.mashups.enabler.widget.WidgetDefinition");
dojo.require("com.ibm.mashups.enabler.DirtyFlagProvider");

//<INTERFACE>
/**
 * This tree model exposes containers and controls. The following applies to the topology:
 * <ul>
 * <li>the root node is a container</li>
 * <li>containers may contain containers</li>
 * <li>containers may contain controls</li>
 * <li>controls and containers must not be mixed within a container</li>
 * </ul>
 * The layout model supports the following strategies:
 * <ul>
 * <li>{@link com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy HiddenMetaDataLoadingStrategy}</li>
 * </ul>
 * 
 * @see com.ibm.mashups.enabler.layout.LayoutContainer
 * @see com.ibm.mashups.enabler.layout.LayoutControl
 *
 * @ibm-api
 * @ibm-module LayoutModel
 */
dojo.declare("com.ibm.mashups.enabler.model.LayoutModel", [com.ibm.mashups.enabler.TreeModelController, com.ibm.mashups.enabler.wire.WireModelProvider, com.ibm.mashups.enabler.DirtyFlagProvider], {

    /**
     * Creates a layout control. Note that the layout control will not appear in the model unless
     * it is inserted using the <code>insert</code> method.<br>
     * <br>
     * <b>Example:</b> To create a layout control based on a widget definition located at a specific URL, 
     * you do the following:<br>
     * &nbsp;&nbsp;<code>var context = {};</code><br>
     * &nbsp;&nbsp;<code>context["type"] = com.ibm.mashups.enabler.layout.Constants.LAYOUT_CONTROL;</code><br>
     * &nbsp;&nbsp;<code>context[com.ibm.mashups.enabler.layout.Constants.WIDGET_DEFINITION_URL] = "http://my/widget/definition.xml";</code><br>
     * &nbsp;&nbsp;<code>layoutModel.create(context);</code>
     * @param {Object} context context with predefined information used for the creation of
     * the layout control. Must not be <code>null</code>. You must specify the following:<br>
     * &nbsp;&nbsp;<b><code>type</code></b> - type of layout entity to create as defined in 
     * &nbsp; {@link com.ibm.mashups.enabler.layout.Constants Constants}; must be one of the 
     * following:<br>
     * &nbsp;&nbsp;&nbsp;LAYOUT_CONTROL<br>
     * <br>
     * &nbsp;&nbsp;In case type equals LAYOUT_CONTROL, you must specify the widget to create 
     * using one of the following keys as defined in &nbsp; {@link com.ibm.mashups.enabler.layout.Constants Constants}<br>
     * &nbsp;&nbsp;<b><code>WIDGET_DEFINITION_ID</code></b> - &nbsp; {@link com.ibm.mashups.enabler.Identifiable Identifiable} 
     * of widget definition to instantiate for this control, or<br>
     * &nbsp;&nbsp;<b><code>WIDGET_DEFINITION_URL</code></b> - URL (string) pointing to a widget definition to instantiate for
     * this control; note that the widget definition will be managed by the backend after this layout control is inserted into a 
     * layout container and the layout model is committed.
     *
     * @return {com.ibm.mashups.enabler.layout.LayoutControl} the created layout control, or <code>null</code>
     * if it can not be created.
     *
     * @see com.ibm.mashups.enabler.widget.WidgetDefinition
     * @see com.ibm.mashups.enabler.layout.Constants
     */
    create: function(context){
    },
    
    /**
     * Returns the root node of the layout model; never <code>null</code>.
     *
     * @return  {com.ibm.mashups.enabler.Deferred} a deferred object used to start this operation. The
     *          return value when executed through the deferred object
     *          is the root &nbsp; {@link com.ibm.mashups.enabler.layout.LayoutNode LayoutNode} of the layout model
     */
    getRoot: function(){
    },
    
    /**
     * Returns whether or not the given node has children.
     *
     * @param   {com.ibm.mashups.enabler.layout.LayoutNode} node the layout node for which to check if it has
     * 			children; must not be <code>null</code>.
     * @return {Boolean} <tt>true</tt> if the specified node has children, <tt>false</tt> otherwise
     */
    hasChildren: function(node){
    },
    
    /**
     * Returns an iterator of the child nodes for the given node.
     *
     * @param 	{com.ibm.mashups.enabler.layout.LayoutNode} node the node object for which to return its children; must not be 
     * 			<code>null</code>.
     * @return  {com.ibm.mashups.enabler.DeferredIterator} deferred iterator of the children for 
     * 			the given node; never <code>null</code>
     */
    getChildren: function(node){
    },
    
    /**
     * Returns the parent of a given node.
     * @param 	{com.ibm.mashups.enabler.layout.LayoutNode} node node object for which to return its parent node; must not
     *          be <code>null</code>
     * @return  {com.ibm.mashups.enabler.Deferred} a deferred object used to start this operation. The
     *          return value when executed through the deferred object
     *          is the parent &nbsp; {@link com.ibm.mashups.enabler.layout.LayoutNode LayoutNode} of the given node, or 
     *          <code>null</code>, if the node has no parent
     */
    getParent: function(node){
    },
    
    /**
     * Indicates if the specified resource is modifiable.
     *
     * @since 2.4
     *
     * @param {com.ibm.mashups.enabler.layout.LayoutNode} node the layout node to check; must not be <code>null</code>
     *
     * @return {Boolean} <code>true</code> if the node is modifiable, <code>false</code> otherwise
     */
    isModifiable: function(node){
    },
    
    /**
     * Indicates if the specified layout node is deletable.
     *
     * @since 2.4
     *
     * @param {com.ibm.mashups.enabler.layout.LayoutNode} node The layout node to check; must not be <code>null</code>
     *
     * @return {Boolean} <code>true</code> if the node is deletable, <code>false</code> otherwise
     */
    isDeletable: function(node){
    },
    
    /**
     * Returns an element of a model (searches throughout the whole tree) with the given ID. May be either a {@link com.ibm.mashups.enabler.layout.LayoutControl LayoutControl} or a {@link com.ibm.mashups.enabler.layout.LayoutContainer LayoutContainer}
     * @param {com.ibm.mashups.enabler.Identifiable} id identifiable or string id of the object to find; 
     * must not be <code>null</code>.
     * @return {com.ibm.mashups.enabler.Deferred} a deferred object used to start this operation. 
     * The return value when executed through the deferred object is
     * the element with the given ID or <code>null</code> if the element cannot be found.
     */
    find: function (id) {
    }    
});
//</INTERFACE>

dojo.require("com.ibm.mm.enabler.layout.LayoutModelImpl");
